# Housekeeping
rm(list=ls())

#Packages 
library(rmarkdown)  # to generate pdf file
library(tinytex)    # to generate pdf file
library(readxl)#     # to read Excel files
library(tseries)#    # to estimate ARMA model
library(zoo)
library(readxl)
library(dplyr)
library(lubridate)
library(e1071)#
library(ggplot2)
library(cowplot)
library(rmgarch)
library(FinTS)
library(vars)
library(MVN)
library(fUnitRoots)
library(MTS)
library(gridExtra)
library(fBasics)
library(interp)
library(forecast)
library(stats)

# Set the working directory
setwd("/Users/nahomberhane/Desktop/Skole/Master thesis/Data")

# Read the Excel file
df_Price <- read_excel("Data_test.xlsx")

reg_x <- data.frame(df_Price$Date, df_Price$Gold, 
                    df_Price$SP500, df_Price$GPR, df_Price$CPI)
reg_y <- data.frame(df_Price$Date, df_Price$Crude, 
                    df_Price$OBX, df_Price$GPR, df_Price$CPI)
reg_z <- data.frame(df_Price$Date, df_Price$Gold,
                    df_Price$OBX, df_Price$GPR, df_Price$CPI)
reg_w <- data.frame(df_Price$Date, df_Price$Crude,
                    df_Price$SP500, df_Price$GPR, df_Price$CPI)

reg_x$Gold_real <- (df_Price$Gold/df_Price$CPI/100)
reg_x$SP500_real <- (df_Price$SP500/df_Price$CPI/100)
reg_y$Crude_real <- (df_Price$Crude/df_Price$CPI/100)
reg_y$OBX_real <- (df_Price$OBX/df_Price$CPI/100)
reg_z$Gold_real <- (df_Price$Gold/df_Price$CPI/100)
reg_z$OBX_real <- (df_Price$OBX/df_Price$CPI/100)
reg_w$Crude_real <- (df_Price$Crude/df_Price$CPI/100)
reg_w$SP500_real <- (df_Price$SP500/df_Price$CPI/100)

# For reg_x
for(col in names(reg_x)[-1]) { # exclude the Date column
  reg_x[[col]] <- log(reg_x[[col]] / lag(reg_x[[col]])) * 100
}

# For reg_y
for(col in names(reg_y)[-1]) { # exclude the Date column
  reg_y[[col]] <- log(reg_y[[col]] / lag(reg_y[[col]])) * 100
}

# For reg_z
for(col in names(reg_z)[-1]) { # exclude the Date column
  reg_z[[col]] <- log(reg_z[[col]] / lag(reg_z[[col]])) * 100
}

# For reg_w
for(col in names(reg_w)[-1]) { # exclude the Date column
  reg_w[[col]] <- log(reg_w[[col]] / lag(reg_w[[col]])) * 100
}

colnames(reg_x) <- c("Date", "Gold", "SP500", "GPR","CPI", "Real_Gold", "Real_SP")
colnames(reg_y) <- c("Date", "Crude", "OBX", "GPR","CPI", "Real_Crude", "Real_OBX")
colnames(reg_z) <- c("Date", "Gold", "OBX", "GPR","CPI", "Real_Gold", "Real_OBX")
colnames(reg_w) <- c("Date", "Crude", "SP500", "GPR","CPI", "Real_Crude", "Real_SP")



# Remove NA rows
reg_x <- na.omit(reg_x)
reg_y <- na.omit(reg_y)
reg_z <- na.omit(reg_z)
reg_w <- na.omit(reg_w)

# Calculate mean, variance, skewness, and excess kurtosis for "Gold"
mean_Gold <- mean(reg_x$Gold)
var_Gold <- var(reg_x$Gold)
skew_Gold <- skewness(reg_x$Gold)
Excess_kurtosis_Gold <- kurtosis(reg_x$Gold)
JB_normality_Gold <- jarque.bera.test(reg_x$Gold)
JB_normality_Gold <- JB_normality_Gold$statistic
ADF_Gold <- adf.test(reg_x$Gold)
ADF_Gold <- ADF_Gold$statistic
KPSS_Gold <- kpss.test(reg_x$Gold)
KPSS_Gold <- KPSS_Gold$statistic
PP_Gold <- pp.test(reg_x$Gold)
PP_Gold <- PP_Gold$statistic

# Calculate mean, variance, skewness, and excess kurtosis for "Crude"
mean_Crude <- mean(reg_y$Crude)
var_Crude <- var(reg_y$Crude)
skew_Crude <- skewness(reg_y$Crude)
Excess_kurtosis_Crude <- kurtosis(reg_y$Crude)
JB_normality_Crude <- jarque.bera.test(reg_y$Crude)
JB_normality_Crude <- JB_normality_Crude$statistic
ADF_Crude <- adf.test(reg_y$Crude)
ADF_Crude <- ADF_Crude$statistic
KPSS_Crude <- kpss.test(reg_y$Crude)
KPSS_Crude <- KPSS_Crude$statistic
PP_Crude <- pp.test(reg_y$Crude)
PP_Crude <- PP_Crude$statistic

# Calculate mean, variance, skewness, and excess kurtosis for "OBX"
mean_OBX <- mean(reg_y$OBX)
var_OBX <- var(reg_y$OBX)
skew_OBX <- skewness(reg_y$OBX)
Excess_kurtosis_OBX <- kurtosis(reg_y$OBX)
JB_normality_OBX <- jarque.bera.test(reg_y$OBX)
JB_normality_OBX <- JB_normality_OBX$statistic
ADF_OBX <- adf.test(reg_y$OBX)
ADF_OBX <- ADF_OBX$statistic
KPSS_OBX <- kpss.test(reg_y$OBX)
KPSS_OBX <- KPSS_OBX$statistic
PP_OBX <- pp.test(reg_y$OBX)
PP_OBX <- PP_OBX$statistic

# Calculate mean, variance, skewness, and excess kurtosis for "SP500"
mean_SP500 <- mean(reg_x$SP500)
var_SP500 <- var(reg_x$SP500)
skew_SP500 <- skewness(reg_x$SP500)
Excess_kurtosis_SP500 <- kurtosis(reg_x$SP500)
JB_normality_SP500 <- jarque.bera.test(reg_x$SP500)
JB_normality_SP500 <- JB_normality_SP500$statistic
ADF_SP500 <- adf.test(reg_x$SP500)
ADF_SP500 <- ADF_SP500$statistic
KPSS_SP500 <- kpss.test(reg_x$SP500)
KPSS_SP500 <- KPSS_SP500$statistic
PP_SP500 <- pp.test(reg_x$SP500)
PP_SP500 <- PP_SP500$statistic

# Calculate mean, variance, skewness, and excess kurtosis for "GPR2"
mean_GPR <- mean(reg_x$GPR)
var_GPR <- var(reg_x$GPR)
skew_GPR <- skewness(reg_x$GPR)
Excess_kurtosis_GPR <- kurtosis(reg_x$GPR)
JB_normality_GPR <- jarque.bera.test(reg_x$GPR)
JB_normality_GPR <- JB_normality_GPR$statistic
ADF_GPR <- adf.test(reg_x$GPR)
ADF_GPR <- ADF_GPR$statistic
KPSS_GPR <- kpss.test(reg_x$GPR)
KPSS_GPR <- KPSS_GPR$statistic
PP_GPR <- pp.test(reg_x$GPR)
PP_GPR <- PP_GPR$statistic

# Create a data frame for the summary table
summary_table <- data.frame(
 Measure = c("Mean", "Variance", "Skewness", "Excess Kurtosis"
              ,"J-B Normality", "ADF", "KPSS", "PP"),
  Gold = c(mean_Gold, var_Gold, skew_Gold, Excess_kurtosis_Gold, 
           JB_normality_Gold, ADF_Gold, KPSS_Gold, PP_Gold),
  SP500 = c(mean_SP500, var_SP500, skew_SP500, Excess_kurtosis_SP500, 
            JB_normality_SP500, ADF_SP500, KPSS_SP500, PP_SP500),
  GPR = c(mean_GPR, var_GPR, skew_GPR, Excess_kurtosis_GPR, 
           JB_normality_GPR, ADF_GPR, KPSS_GPR, PP_GPR),
  Crude = c(mean_Crude, var_Crude, skew_Crude, Excess_kurtosis_Crude, 
            JB_normality_Crude, ADF_Crude, KPSS_Crude, PP_Crude),
  OBX = c(mean_OBX, var_OBX, skew_OBX, Excess_kurtosis_OBX, 
          JB_normality_OBX, ADF_OBX, KPSS_OBX, PP_OBX)
)
print(summary_table)

##################################################################
################# Timeseries plot of all assets ##################
##################################################################

##############################################
#################### GOLD ####################
##############################################

x <- data.frame(
  x = reg_x$Date,
  y = reg_x$Gold
) 

p1 <- (ggplot(x, aes(x = x, y = y)) +
  geom_line(color = "black") +
  scale_x_datetime(date_breaks = "5 year", date_labels = "%Y") +
  labs(x = "Date", y = "Return", 
       title = "Gold return") +
  theme_minimal())


##############################################
#################### SP 500 ##################
##############################################

x <- data.frame(
  x = reg_x$Date,
  y = reg_x$SP500
) 

p2 <- (ggplot(x, aes(x = x, y = y)) +
  geom_line(color = "black") +
  scale_x_datetime(date_breaks = "5 year", date_labels = "%Y") +
  labs(x = "Date", y = "Return", 
       title = "SP500 return") +
  theme_minimal())

##############################################
#################### Crude  ##################
##############################################

x <- data.frame(
  x = reg_w$Date,
  y = reg_w$Crude
) 

p3 <- (ggplot(x, aes(x = x, y = y)) +
  geom_line(color = "black") +
  scale_x_datetime(date_breaks = "5 year", date_labels = "%Y") +
  labs(x = "Date", y = "Return", 
       title = "Crude return") +
  theme_minimal())

##############################################
#################### OBX  ####################
##############################################

x <- data.frame(
  x = reg_y$Date,
  y = reg_y$OBX
) 

p4 <- (ggplot(x, aes(x = x, y = y)) +
  geom_line(color = "black") +
  scale_x_datetime(date_breaks = "5 year", date_labels = "%Y") +
  labs(x = "Date", y = "Return", 
       title = "OBX return") +
  theme_minimal())

##############################################
#################### GPR  ####################
##############################################

x <- data.frame(
  x = reg_x$Date,
  y = reg_x$GPR
) 

p5 <- (ggplot(x, aes(x = x, y = y)) +
         geom_line(color = "black") +
         scale_x_datetime(date_breaks = "5 year", date_labels = "%Y") +
         labs(x = "Date", y = "Index Value", 
              title = "GPR Chart") +
         theme_minimal())

grid.arrange(p1, p3, nrow=2)
grid.arrange(p2, p4, nrow=2)
plot (p5)
##############################################################################
################# Calculating Timevarying correlation ########################
##############################################################################

##############################################
################# GOLDxSP ####################
##############################################


# Create a data frame with selected columns
df <- data.frame(
  Date = reg_x$Date,
  Gold = reg_x$Real_Gold,
  SP500 = reg_x$Real_SP
)

# Calculate time-varying correlation
window_size <- 10  # Size of the rolling window
correlation_Gold_SP500 <- numeric(nrow(df))
for (i in window_size:nrow(df)) {
  correlation_Gold_SP500[i] <- cor(df$Gold[(i-window_size+1):i], df$SP500[(i-window_size+1):i])
}

# Create a new data frame with Date, Correlation, and GPR2 columns
Time_vary_corr_Gold_SP500 <- data.frame(
  Date = reg_x$Date[(window_size + 1):nrow(reg_x)],
  Correlation = correlation_Gold_SP500[(window_size + 1):nrow(reg_x)],
  GPR = reg_x$GPR[(window_size + 1):nrow(reg_x)]
)

# Remove rows with missing values
Time_vary_corr_Gold_SP500 <- Time_vary_corr_Gold_SP500[complete.cases(Time_vary_corr_Gold_SP500), ]

# Calculate the number of dates
ndates <- nrow(Time_vary_corr_Gold_SP500)

##############################################
################# CrudexOBX ##################
##############################################


# Create a data frame with selected columns
df <- data.frame(
  Date = reg_y$Date,
  Crude = reg_y$Real_Crude,
  OBX = reg_y$Real_OBX
)

# Calculate time-varying correlation
window_size <- 10  # Size of the rolling window
correlation_Crude_OBX <- numeric(nrow(df))
for (i in window_size:nrow(df)) {
  correlation_Crude_OBX[i] <- cor(df$Crude[(i-window_size+1):i], df$OBX[(i-window_size+1):i])
}

# Create a new data frame with Date, Correlation, and GPR2 columns
Time_vary_corr_Crude_OBX <- data.frame(
  Date = reg_y$Date[(window_size + 1):nrow(reg_y)],
  Correlation = correlation_Crude_OBX[(window_size + 1):nrow(reg_y)],
  GPR = reg_y$GPR[(window_size + 1):nrow(reg_y)]
)

# Remove rows with missing values
Time_vary_corr_Crude_OBX <- Time_vary_corr_Crude_OBX[complete.cases(Time_vary_corr_Crude_OBX), ]

# Calculate the number of dates
ndates <- nrow(Time_vary_corr_Crude_OBX)

##############################################
################# GOLDxOBX ###################
##############################################


# Create a data frame with selected columns
df <- data.frame(
  Date = reg_z$Date,
  Gold = reg_z$Real_Gold,
  OBX = reg_z$Real_OBX
)

# Calculate time-varying correlation
window_size <- 10  # Size of the rolling window
correlation_Gold_OBX <- numeric(nrow(df))
for (i in window_size:nrow(df)) {
  correlation_Gold_OBX[i] <- cor(df$Gold[(i-window_size+1):i], df$OBX[(i-window_size+1):i])
}

# Create a new data frame with Date, Correlation, and GPR2 columns
Time_vary_corr_Gold_OBX <- data.frame(
  Date = reg_y$Date[(window_size + 1):nrow(reg_z)],
  Correlation = correlation_Gold_OBX[(window_size + 1):nrow(reg_z)],
  GPR = reg_z$GPR[(window_size + 1):nrow(reg_z)]
)

# Remove rows with missing values
Time_vary_corr_Gold_OBX <- Time_vary_corr_Gold_OBX[complete.cases(Time_vary_corr_Gold_OBX), ]

# Calculate the number of dates
ndates <- nrow(Time_vary_corr_Gold_OBX)


##############################################
################# CrudexSP ###################
##############################################


# Create a data frame with selected columns
df <- data.frame(
  Date = reg_w$Date,
  Crude = reg_w$Real_Crude,
  SP500 = reg_w$Real_SP
)

# Calculate time-varying correlation
window_size <- 10  # Size of the rolling window
correlation_Crude_SP500 <- numeric(nrow(df))
for (i in window_size:nrow(df)) {
  correlation_Crude_SP500[i] <- cor(df$Crude[(i-window_size+1):i], df$SP500[(i-window_size+1):i])
}

# Create a new data frame with Date, Correlation, and GPR2 columns
Time_vary_corr_Crude_SP500 <- data.frame(
  Date = reg_w$Date[(window_size + 1):nrow(reg_w)],
  Correlation = correlation_Crude_SP500[(window_size + 1):nrow(reg_w)],
  GPR = reg_w$GPR[(window_size + 1):nrow(reg_w)]
)

# Remove rows with missing values
Time_vary_corr_Crude_SP500 <- Time_vary_corr_Crude_SP500[complete.cases(Time_vary_corr_Crude_SP500), ]

# Calculate the number of dates
ndates <- nrow(Time_vary_corr_Crude_SP500)


###########################################################################
################# Plotting Timevarying correlation ########################
###########################################################################
#################################################
############# Plotting GoldxSP ##################
#################################################

# Create a data frame to hold the data
dfx <- data.frame(x = Time_vary_corr_Gold_SP500$Date, 
                 y1 = Time_vary_corr_Gold_SP500$Correlation, 
                 y2 = Time_vary_corr_Gold_SP500$GPR)

# Define a scaling factor
scale_factor <- 400

# Create the base plot
p1 <- ggplot(dfx, aes(x = x)) +
  geom_line(aes(y = y1 * scale_factor, colour = "Correlation")) +
  scale_y_continuous(name = "GPR",
                     sec.axis = sec_axis(~ . / scale_factor, name = "Correlation")) +
  geom_bar(aes(y = y2, colour = "GPR"), stat="identity") +
  labs(x = "Date", colour = "Series", title = "Correlation of Gold with SP500 and GPR variation") +
  theme_minimal() +
  theme(legend.position = "bottom")


###################################################
############# Plotting CrudexOBX ##################
###################################################

# Create a data frame to hold the data
dfy <- data.frame(x = Time_vary_corr_Crude_OBX$Date, 
                 y1 = Time_vary_corr_Crude_OBX$Correlation, 
                 y2 = Time_vary_corr_Crude_OBX$GPR)

# Define a scaling factor
scale_factor <- 400

# Create the base plot
p2 <- ggplot(dfy, aes(x = x)) +
  geom_line(aes(y = y1 * scale_factor, colour = "Correlation")) +
  scale_y_continuous(name = "GPR",
                     sec.axis = sec_axis(~ . / scale_factor, name = "Correlation")) +
  geom_bar(aes(y = y2, colour = "GPR"),stat="identity") +
  labs(x = "Date", colour = "Series", title = "Correlation of Crude with OBX and GPR variation") +
  theme_minimal() +
  theme(legend.position = "bottom")



###################################################
############# Plotting GoldxOBX ###################
###################################################

# Create a data frame to hold the data
dfz <- data.frame(x = Time_vary_corr_Gold_OBX$Date, 
                 y1 = Time_vary_corr_Gold_OBX$Correlation, 
                 y2 = Time_vary_corr_Gold_OBX$GPR)

# Define a scaling factor
scale_factor <- 400

# Create the base plot
p3 <- ggplot(dfz, aes(x = x)) +
  geom_line(aes(y = y1 * scale_factor, colour = "Correlation")) +
  scale_y_continuous(name = "GPR",
                     sec.axis = sec_axis(~ . / scale_factor, name = "Correlation")) +
  geom_bar(aes(y = y2, colour = "GPR"), stat="identity") +
  labs(x = "Date", colour = "Series", title = "Correlation of Gold with OBX and GPR variation") +
  theme_minimal() +
  theme(legend.position = "bottom")




###################################################
############# Plotting CrudexSP ###################
###################################################

# Create a data frame to hold the data
dfw <- data.frame(x = Time_vary_corr_Crude_SP500$Date, 
                 y1 = Time_vary_corr_Crude_SP500$Correlation, 
                 y2 = Time_vary_corr_Crude_SP500$GPR)

# Define a scaling factor
scale_factor <- 400

# Create the base plot
p4 <- ggplot(dfw, aes(x = x)) +
  geom_line(aes(y = y1 * scale_factor, colour = "Correlation")) +
  scale_y_continuous(name = "GPR",
                     sec.axis = sec_axis(~ . / scale_factor, name = "Correlation")) +
  geom_bar(aes(y = y2, colour = "GPR"), stat="identity") +
  labs(x = "Date", colour = "Series", title = "Correlation of Crude with SP500 and GPR variation") +
  theme_minimal() +
  theme(legend.position = "bottom")


grid.arrange (p1,p2,p3,p4, ncol =2, nrow =2)

###############################################################
################# Bivariate ARCH and Normality ################
###############################################################
x <- cbind(reg_x$Gold, reg_x$SP500)
colnames(x) <- c("Gold", "SP500")
x_GPR <- reg_x$GPR

Arch_GOLDSP = ArchTest (x, lags=12)
print(Arch_GOLDSP)

Normality_GOLDSP = jarqueberaTest(x)
print(Normality_GOLDSP)

y <- cbind(reg_y$Crude, reg_y$OBX)
colnames(y) <- c("Crude", "OBX")
y_GPR <- reg_y$GPR

Arch_CRUDEOBX = ArchTest (y, lags=12)
print(Arch_CRUDEOBX)

Normality_CRUDEOBX = jarqueberaTest(y)
print(Normality_CRUDEOBX)

z <- cbind(reg_z$Gold, reg_z$OBX)
colnames(z) <- c("Gold", "OBX")
z_GPR <- reg_z$GPR

Arch_GOLDOBX = ArchTest (z, lags=12)
print(Arch_GOLDOBX)

Normality_GOLDOBX = jarqueberaTest(z)
print(Normality_GOLDOBX)

w <- cbind(reg_w$Crude, reg_w$SP)
colnames(w) <- c("Crude", "SP500")
w_GPR <- reg_w$GPR

Arch_CRUDESP = ArchTest (w, lags=12)
print(Arch_CRUDESP)

Normality_CRUDESP = jarqueberaTest(w)
print(Normality_CRUDESP)

###############################################################
#################### DCC GARCH FIT PARAMETERS #################
###############################################################

###############################################################
######################## GOLD & SP500 #########################
###############################################################
x <- cbind(reg_x$Gold, reg_x$SP500)
colnames(x) <- c("Gold", "SP500")

x_GPR <- reg_x$GPR

x <- na.omit(x, x_GPR)

# Specify the univariate GARCH models for gold and sp500
spec.n = multispec(replicate(2, ugarchspec(mean.model = list(armaOrder = c(1, 4), include.mean = TRUE, external.regressors = as.matrix((x_GPR))), 
                                           variance.model = list(model = "sGARCH", garchOrder = c(1, 1), external.regressors = as.matrix((x_GPR))))))
multf = multifit (spec.n, x, solver="hybrid")

test = dccspec(uspec = spec.n, lag = 1,
               dccOrder = c(1,1), distribution= c ("mvnorm"))

# Fit the univariate GARCH models
fitx = dccfit(test, data=x, fit.control = list(eval.se =TRUE), fit =multf)
print(fitx)

volatilityx <- sigma(fitx)
print(volatilityx)

residuals = residuals(multf, standardize = TRUE)
archTest1 = ArchTest(residuals[,])
archTest2 = ArchTest(residuals[,2])
print(archTest1)
print(archTest2)


std_resid <- fitx@mfit$stdresid
colnames(std_resid) <- c("Gold", "SP500")
plot(std_resid)

covx <- rcov(fitx)
plot(covx)


###############################################################
######################## CRUDE & OBX ##########################
###############################################################

y <- cbind(reg_y$Crude, reg_y$OBX)
colnames(y) <- c("Crude", "OBX")

y_GPR <- reg_y$GPR

y <- na.omit(y, y_GPR)

# Specify the univariate GARCH models for gold and sp500
spec.n = multispec(replicate(2, ugarchspec(mean.model = list(armaOrder = c(1, 0), include.mean = TRUE, external.regressors = as.matrix((y_GPR))), 
                                           variance.model = list(model = "sGARCH", garchOrder = c(1, 1), external.regressors = as.matrix((y_GPR))))))
multf = multifit (spec.n, y, solver="hybrid")

test = dccspec(uspec = spec.n, lag = 1,
               dccOrder = c(1,1), distribution= c ("mvnorm"))

# Fit the univariate GARCH models
fity = dccfit(test, data=y, fit.control = list(eval.se =TRUE), fit =multf)
print(fity)

volatilityy <- sigma(fity)
print(volatilityy)

residuals = residuals(multf, standardize = TRUE)
archTest1 = ArchTest(residuals[,])
archTest2 = ArchTest(residuals[,2])
print(archTest1)
print(archTest2)

std_resid <- fity@mfit$stdresid
colnames(std_resid) <- c("Crude", "OBX")
plot(std_resid)

covy <- rcov(fity)
plot(covy)


###############################################################
######################## GOLD & OBX ###########################
###############################################################

z <- cbind(reg_z$Gold, reg_z$OBX)
z <- na.omit(z)

colnames(z) <- c("Gold", "OBX")

z_GPR <- reg_z$GPR

z <- na.omit(z, z_GPR)

# Specify the univariate GARCH models for gold and sp500
spec.n = multispec(replicate(2, ugarchspec(mean.model = list(armaOrder = c(1, 0), include.mean = TRUE, external.regressors = as.matrix((z_GPR))), 
                                           variance.model = list(model = "sGARCH", garchOrder = c(1, 1), external.regressors = as.matrix((z_GPR))))))
multf = multifit (spec.n, z, solver="hybrid")

test = dccspec(uspec = spec.n, lag = 1,
               dccOrder = c(1,1), distribution= c ("mvnorm"))

# Fit the univariate GARCH models
fitz = dccfit(test, data=z, fit.control = list(eval.se =TRUE), fit =multf)
print(fitz)

volatilityz <- sigma(fitz)
print(volatilityz)

residuals = residuals(multf, standardize = TRUE)
archTest1 = ArchTest(residuals[,])
archTest2 = ArchTest(residuals[,2])
print(archTest1)
print(archTest2)


std_resid <- fitz@mfit$stdresid
colnames(std_resid) <- c("Gold", "OBX")
plot(std_resid)

covz <- rcov(fitz)
plot(covz)

###############################################################
######################## Crude & SP ###########################
###############################################################

w <- cbind(reg_w$Crude, reg_w$SP500)
w <- na.omit(w)

colnames(w) <- c("Crude", "SP500")

w_GPR <- reg_w$GPR

w <- na.omit(w, w_GPR)

# Specify the univariate GARCH models for gold and sp500
spec.n = multispec(replicate(2, ugarchspec(mean.model = list(armaOrder = c(1, 0), include.mean = TRUE, external.regressors = as.matrix((w_GPR))), 
                                           variance.model = list(model = "sGARCH", garchOrder = c(1, 1), external.regressors = as.matrix((w_GPR))))))
multf = multifit (spec.n, w, solver="hybrid")

test = dccspec(uspec = spec.n, lag = 1,
               dccOrder = c(1,1), distribution= c ("mvnorm"))

# Fit the univariate GARCH models
fitw = dccfit(test, data=w, fit.control = list(eval.se =TRUE), fit =multf)
print(fitw)

volatilityw <- sigma(fitw)
print(volatilityw)

residuals = residuals(multf, standardize = TRUE)
archTest1 = ArchTest(residuals[,])
archTest2 = ArchTest(residuals[,2])
print(archTest1)
print(archTest2)


std_resid <- fitw@mfit$stdresid
colnames(std_resid) <- c("Crude", "SP500")
plot(std_resid)

covw <- rcov(fitw)
plot(covw)

#################################################################
######################## DCC Forecast ###########################
#################################################################
#n.ahead = 100
#n.roll = 24
#dcc_cast <- dccforecast(fit1, n.ahead = 100)
#rf <- dcc_cast@mforecast$R
#rf
#str(rf)


#corf_GOLD <- rf[[1]][1,2,]
#corf_SP500 <- rf[[1]][1,3,]
#corf_BG <- rf[[1]][2,3,]

#par(mfrow=c(3,1))
#c_IB <- c(tail(cor1[1,2,],50), rep (NA,50))
#cf_IB <- c(rep(NA,50),corf_IB)
#plot(c_IB,type="l", main = "Correlation IBM and BP")
#lines(cf_IB,type="l",col="orange")

#c_IG <- c(tail(cor1[1,3,],50), rep (NA,50))
#cf_IG <- c(rep(NA,50),corf_IG)
#plot(c_IG,type="l", main = "Correlation IBM and Google")
#lines(cf_IG,type="l",col="orange")

#c_BG <- c(tail(cor1[2,3,],50), rep (NA,50))
#cf_BG <- c(rep(NA,50),corf_BG)
#plot(c_BG,type="l", main = "Correlation BP and Google")
#lines(cf_BG,type="l",col="orange")

###############################################################
#################### Copula GARCH FIT PARAMETERS ##############
###############################################################

###############################################################
####################### GOLD & SP500 ##########################
###############################################################

multif2 <- multispec(replicate(2, ugarchspec(mean.model = list(armaOrder = c(1, 0), include.mean = TRUE,external.regressors = as.matrix(x_GPR)), 
                                             variance.model = list(model = "sGARCH", garchOrder = c(1,1), external.regressors = as.matrix(x_GPR)))))
multf <- multifit (multif2, x)
spec.m = cgarchspec(uspec = multif2, lag=1, external.regressors = as.matrix(x_GPR),  dccOrder = c(1,1), 
                    distribution.model = list (copula = c("mvt"), time.varying = TRUE))
fit2 <- cgarchfit(spec.m, data = x, fit.control = list(eval.se=TRUE), fit = multf)

print(fit2)

###############################################################
####################### CRUDE & OBX ###########################
###############################################################

multif2 <- multispec(replicate(2, ugarchspec(mean.model = list(armaOrder = c(1, 0), include.mean = TRUE, external.regressors = as.matrix(y_GPR)), 
                                             variance.model = list(model = "sGARCH", garchOrder = c(1,1),external.regressors = as.matrix(y_GPR)))))
multf <- multifit (multif2, y)
spec.m = cgarchspec(uspec = multif2, lag=1, external.regressors = as.matrix(y_GPR),  dccOrder = c(1,1), 
                    distribution.model = list (copula = c("mvt"), time.varying = TRUE))
fit2 <- cgarchfit(spec.m, data = y, fit.control = list(eval.se=TRUE), fit = multf)

print(fit2)

###############################################################
####################### GOLD & OBX ############################
###############################################################

multif2 <- multispec(replicate(2, ugarchspec(mean.model = list(armaOrder = c(1, 0), include.mean = TRUE,external.regressors = as.matrix(z_GPR)), 
                                             variance.model = list(model = "sGARCH", garchOrder = c(1,1), external.regressors = as.matrix(z_GPR)))))
multf <- multifit (multif2, z)
spec.m = cgarchspec(uspec = multif2, lag=1, external.regressors = as.matrix(z_GPR),  dccOrder = c(1,1), 
                    distribution.model = list (copula = c("mvt"), time.varying = TRUE))
fit2 <- cgarchfit(spec.m, data = z, fit.control = list(eval.se=TRUE), fit = multf)

print(fit2)

###############################################################
####################### Crude & SP ############################
###############################################################

multif2 <- multispec(replicate(2, ugarchspec(mean.model = list(armaOrder = c(1, 0), include.mean = TRUE,external.regressors = as.matrix(w_GPR)), 
                                             variance.model = list(model = "sGARCH", garchOrder = c(1,1), external.regressors = as.matrix(w_GPR)))))
multf <- multifit (multif2, w)
spec.m = cgarchspec(uspec = multif2, lag=1, external.regressors = as.matrix(w_GPR),  dccOrder = c(1,1), 
                    distribution.model = list (copula = c("mvt"), time.varying = TRUE))
fit2 <- cgarchfit(spec.m, data = w, fit.control = list(eval.se=TRUE), fit = multf)

print(fit2)


#################################################################
######################## DCC Forecast ###########################
#################################################################

###############################################################
####################### GOLD & SP #############################
###############################################################

df<- data.frame(correlation_Gold_SP500, reg_x$GPR)

df <- na.omit(df)

colnames(df) <- c("Corr", "GPR")

# Calculate time-varying correlation
window_size <- 10  # Size of the rolling window
correlation_GPR <- numeric(nrow(df))
for (i in window_size:nrow(df)) {
  correlation_GPR[i] <- cor(df$Corr[(i-window_size+1):i], df$GPR[(i-window_size+1):i])
}

datax1 <- correlation_GPR
correlation_GPR <- na.omit(correlation_GPR)
datax1 <- ts(datax1[1:419])
datax2 <- ts(correlation_GPR[420:455])

mdlx1 <- auto.arima(datax1, max.p=3, max.q=3, max.d=1, 
                  stepwise=FALSE, approximation=FALSE, allowdrift=TRUE, allowmean=TRUE)


datax12 <- ts(c(datax1,datax2))
mdlx2 <- Arima(datax12, model=mdlx1)
plot(datax1, xlim=c(0,455))
lines(window(fitted(mdlx2), start=420), col='red')


############################
############################
############################

datax3 <- correlation_Gold_SP500
correlation_Gold_SP500 <- na.omit(correlation_Gold_SP500)
datax3 <- ts(datax3[1:419])
datax4 <- ts(correlation_Gold_SP500[420:455])

mdlx3 <- auto.arima(datax3, max.p=3, max.q=3, max.d=1, 
                  stepwise=FALSE, approximation=FALSE, allowdrift=TRUE, allowmean=TRUE)


datax34 <- ts(c(datax3,datax4))
mdlx4 <- Arima(datax34, model=mdlx3)
plot(datax3, type="l", xlim=c(0,455), main="Correlation of Gold with SP500", ylab = "Correlation")
lines(window(fitted(mdlx4), start=420), col="green")
lines(window(fitted(mdlx2), start=420), col='blue')
legend("bottomleft", legend = c("W/GPR", "Correlation"), col = c("blue", "green"), lty = 1, box.lty=0)



#################################################################
####################### Crude & OBX #############################
#################################################################


df<- data.frame(correlation_Crude_OBX, reg_y$GPR)

df <- na.omit(df)

colnames(df) <- c("Corr", "GPR")

# Calculate time-varying correlation
window_size <- 10  # Size of the rolling window
correlation_GPR <- numeric(nrow(df))
for (i in window_size:nrow(df)) {
  correlation_GPR[i] <- cor(df$Corr[(i-window_size+1):i], df$GPR[(i-window_size+1):i])
}

datay <- correlation_GPR
correlation_GPR <- na.omit(correlation_GPR)
datay1 <- ts(datay[1:286])
datay2 <- ts(correlation_GPR[287:323])

mdly1 <- auto.arima(datay1, max.p=3, max.q=3, max.d=1, 
                    stepwise=FALSE, approximation=FALSE, allowdrift=TRUE, allowmean=TRUE)


datay12 <- ts(c(datay1,datay2))
mdly2 <- Arima(datay12, model=mdly1)
plot(datay1, type="l", xlim=c(0,323))
lines(window(fitted(mdly2), start=287, end=323), col='red')


############################
############################
############################

datay3 <- correlation_Crude_OBX
correlation_Crude_OBX <- na.omit(correlation_Crude_OBX)
datay3 <- ts(datay3[1:286])
datay4 <- ts(correlation_Crude_OBX[287:323])

mdly3 <- auto.arima(datay3, max.p=3, max.q=3, max.d=1, 
                    stepwise=FALSE, approximation=FALSE, allowdrift=TRUE, allowmean=TRUE)


datay34 <- ts(c(datay3,datay4))
mdly4 <- Arima(datay34, model=mdly3)
plot(datay3, type="l", xlim=c(0,323), main="Correlation of Crude with OBX", ylab = "Correlation")
lines(window(fitted(mdly4), start=287), col="green")
lines(window(fitted(mdly2), start=287), col='blue')
legend("bottomright", legend = c("W/GPR", "Correlation"), col = c("blue", "green"), lty = 1, box.lty=0)




#################################################################
####################### Gold & OBX ##############################
#################################################################


df<- data.frame(correlation_Gold_OBX, reg_z$GPR)

df <- na.omit(df)

colnames(df) <- c("Corr", "GPR")

# Calculate time-varying correlation
window_size <- 10  # Size of the rolling window
correlation_GPR <- numeric(nrow(df))
for (i in window_size:nrow(df)) {
  correlation_GPR[i] <- cor(df$Corr[(i-window_size+1):i], df$GPR[(i-window_size+1):i])
}

dataz <- correlation_GPR
correlation_GPR <- na.omit(correlation_GPR)
dataz1 <- ts(dataz[1:286])
dataz2 <- ts(correlation_GPR[287:323])


mdlz1 <- auto.arima(dataz1, max.p=3, max.q=3, max.d=1, 
                    stepwise=FALSE, approximation=FALSE, allowdrift=TRUE, allowmean=TRUE)


dataz12 <- ts(c(dataz1,dataz2))
mdlz2 <- Arima(dataz12, model=mdlz1)
plot(dataz1, type="l", xlim=c(0,323))
lines(window(fitted(mdlz2), start=287, end=323), col='red')


############################
############################
############################

dataz3 <- correlation_Crude_OBX
correlation_Crude_OBX <- na.omit(correlation_Crude_OBX)
dataz3 <- ts(dataz3[1:286])
dataz4 <- ts(correlation_Crude_OBX[287:323])

mdlz3 <- auto.arima(dataz3, max.p=3, max.q=3, max.d=1, 
                    stepwise=FALSE, approximation=FALSE, allowdrift=TRUE, allowmean=TRUE)


dataz34 <- ts(c(dataz3,dataz4))
mdlz4 <- Arima(dataz34, model=mdlz3)
plot(dataz3, type="l", xlim=c(0,323), main="Correlation of Gold with OBX", ylab = "Correlation")
lines(window(fitted(mdlz4), start=287), col="green")
lines(window(fitted(mdlz2), start=287), col='blue')
legend("bottomright", legend = c("W/GPR", "Correlation"), col = c("blue", "green"), lty = 1, box.lty=0)

dates <- seq(as.Date("1985/01/01"), as.Date("2022/11/01"), by = "month")
plot(dates, datax34, type = "l", xaxt = "n", xlab = "Date", ylab = "Correlation")
axis.Date(1, at = seq(min(dates), max(dates), by = "year"), format = "%b-%Y")

###############################################################
####################### Crude & SP ############################
###############################################################

df<- data.frame(correlation_Crude_SP500, reg_w$GPR)

df <- na.omit(df)

colnames(df) <- c("Corr", "GPR")

# Calculate time-varying correlation
window_size <- 10  # Size of the rolling window
correlation_GPR <- numeric(nrow(df))
for (i in window_size:nrow(df)) {
  correlation_GPR[i] <- cor(df$Corr[(i-window_size+1):i], df$GPR[(i-window_size+1):i])
}

dataw1 <- correlation_GPR
correlation_GPR <- na.omit(correlation_GPR)
dataw1 <- ts(dataw1[1:419])
dataw2 <- ts(correlation_GPR[420:455])

mdlw1 <- auto.arima(dataw1, max.p=3, max.q=3, max.d=1, 
                    stepwise=FALSE, approximation=FALSE, allowdrift=TRUE, allowmean=TRUE)


dataw12 <- ts(c(dataw1,dataw2))
mdlw2 <- Arima(dataw12, model=mdlw1)
plot(dataw1, xlim=c(0,455))
lines(window(fitted(mdlw2), start=420), col='red')


############################
############################
############################

dataw3 <- correlation_Gold_SP500
correlation_Gold_SP500 <- na.omit(correlation_Gold_SP500)
dataw3 <- ts(dataw3[1:419])
dataw4 <- ts(correlation_Gold_SP500[420:455])

mdlw3 <- auto.arima(dataw3, max.p=3, max.q=3, max.d=1, 
                    stepwise=FALSE, approximation=FALSE, allowdrift=TRUE, allowmean=TRUE)


dataw34 <- ts(c(dataw3,dataw4))
mdlw4 <- Arima(dataw34, model=mdlw3)
plot(dataw3, type="l", xlim=c(0,455), main="Correlation of Crude with SP500", ylab = "Correlation")
lines(window(fitted(mdlw4), start=420), col="green")
lines(window(fitted(mdlw2), start=420), col='blue')
legend("bottomright", legend = c("W/GPR", "Correlation"), col = c("blue", "green"), lty = 1, box.lty=0)


###############################################################
#################### ROLLING RANK CORRELATION #################
###############################################################

###############################################################
####################### GOLD & SP #############################
###############################################################
# Convert data to zoo object for easy handling of time series
x <- cbind(reg_x$Gold, reg_x$SP500, reg_x$Date)
colnames(x) <- c("Gold", "SP500", "Date")

x_GPR <- reg_x$GPR

x <- na.omit(x)

data_zoo <- x

# Define window size
window_size <- 6

# Calculate sliding rank correlations
rank_correlations <- rollapply(data_zoo, window_size, function(x) cor(x[,1], x[,2], method = "spearman"), by.column = FALSE)

# Define number of bootstrap samples
n_bootstrap <- 1000

# Bootstrap the rank correlations
bootstrap_samples <- replicate(n_bootstrap, sample(rank_correlations, replace = TRUE))

# Calculate bootstrap means and confidence intervals
bootstrap_mean <- colMeans(bootstrap_samples)
bootstrap_ci <- apply(bootstrap_samples, 2, function(x) quantile(x, c(0.025, 0.975)))

# Print the bootstrap mean and confidence intervals
print(bootstrap_mean)
print(bootstrap_ci)

d <- data.frame(reg_x$Date)
d <- d [6:455,]

# Convert rank correlations to a data frame
rank_correlations_df <- data.frame(
  Time = d,
  Correlation = coredata(rank_correlations)
)


# Plot the sliding rank correlations
ggplot(rank_correlations_df, aes(x = Time, y = Correlation)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Rank Correlation", 
       title = " Rank correlation between Gold and SP500") +
  theme_minimal()

###############################################################
####################### Crude & OBX ###########################
###############################################################

# Convert data to zoo object for easy handling of time series
y <- cbind(reg_y$Crude, reg_y$OBX)
colnames(y) <- c("Crude", "OBX")
data_zoo <- y
# Define window size
window_size <- 6

# Calculate sliding rank correlations
rank_correlations <- rollapply(data_zoo, window_size, function(y) cor(y[,1], y[,2], method = "spearman"), by.column = FALSE)

# Define number of bootstrap samples
n_bootstrap <- 1000

# Bootstrap the rank correlations
bootstrap_samples <- replicate(n_bootstrap, sample(rank_correlations, replace = TRUE))

# Calculate bootstrap means and confidence intervals
bootstrap_mean <- colMeans(bootstrap_samples)
bootstrap_ci <- apply(bootstrap_samples, 2, function(y) quantile(y, c(0.025, 0.975)))

# Print the bootstrap mean and confidence intervals
print(bootstrap_mean)
print(bootstrap_ci)

d <- data.frame(reg_y$Date)
d <- d [6:323,]

# Convert rank correlations to a data frame
rank_correlations_df <- data.frame(
  Time = d,
  Correlation = coredata(rank_correlations)
)

# Plot the sliding rank correlations
ggplot(rank_correlations_df, aes(x = Time, y = Correlation)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Rank Correlation", 
       title = " Rank correlation between Crude and OBX") +
  theme_minimal()


###############################################################
####################### GOLD & OBX ############################
###############################################################

# Convert data to zoo object for easy handling of time series
z <- cbind(reg_z$Gold, reg_z$OBX)
colnames(z) <- c("Gold", "OBX")
data_zoo <- z
# Define window size
window_size <- 6

# Calculate sliding rank correlations
rank_correlations <- rollapply(data_zoo, window_size, function(z) cor(z[,1], z[,2], method = "spearman"), by.column = FALSE)

# Define number of bootstrap samples
n_bootstrap <- 1000

# Bootstrap the rank correlations
bootstrap_samples <- replicate(n_bootstrap, sample(rank_correlations, replace = TRUE))

# Calculate bootstrap means and confidence intervals
bootstrap_mean <- colMeans(bootstrap_samples)
bootstrap_ci <- apply(bootstrap_samples, 2, function(z) quantile(z, c(0.025, 0.975)))

# Print the bootstrap mean and confidence intervals
print(bootstrap_mean)
print(bootstrap_ci)

d <- data.frame(reg_z$Date)
d <- d [6:323,]

# Convert rank correlations to a data frame
rank_correlations_df <- data.frame(
  Time = d,
  Correlation = coredata(rank_correlations)
)

# Plot the sliding rank correlations
ggplot(rank_correlations_df, aes(x = Time, y = Correlation)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Rank Correlation", 
       title = " Rank correlation between Gold and OBX") +
  theme_minimal()


###############################################################
####################### Crude & SP ############################
###############################################################

# Convert data to zoo object for easy handling of time series
w <- cbind(reg_w$Crude, reg_w$SP500)
colnames(w) <- c("Crude", "SP500")
data_zoo <- w
# Define window size
window_size <- 6

# Calculate sliding rank correlations
rank_correlations <- rollapply(data_zoo, window_size, function(w) cor(w[,1], w[,2], method = "spearman"), by.column = FALSE)

# Define number of bootstrap samples
n_bootstrap <- 1000

# Bootstrap the rank correlations
bootstrap_samples <- replicate(n_bootstrap, sample(rank_correlations, replace = TRUE))

# Calculate bootstrap means and confidence intervals
bootstrap_mean <- colMeans(bootstrap_samples)
bootstrap_ci <- apply(bootstrap_samples, 2, function(w) quantile(w, c(0.025, 0.975)))

# Print the bootstrap mean and confidence intervals
print(bootstrap_mean)
print(bootstrap_ci)

d <- data.frame(reg_y$Date)
d <- d [6:455,]

# Convert rank correlations to a data frame
rank_correlations_df <- data.frame(
  Time = d,
  Correlation = coredata(rank_correlations)
)

# Plot the sliding rank correlations
ggplot(rank_correlations_df, aes(x = Time, y = Correlation)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Rank Correlation", 
       title = " Rank correlation between Crude and SP500") +
  theme_minimal()


###############################################################
########################## PORTFOLIO ##########################
###############################################################

###############################################################
####################### Gold & SP #############################
###############################################################

Cov_Gold <- volatilityx[,1]
Cov_SP500 <- volatilityx [,2]
Cov_uni <- abs(covx[1,2,])

Hedge_ratio_Gold <- Cov_uni / Cov_SP500
Hedge_ratio_SP500 <- Cov_uni / Cov_Gold

mean_hedge_gold <- mean(Hedge_ratio_Gold)
stdev_hedge_gold <- stdev(Hedge_ratio_Gold)
min_hedge_gold <- min(Hedge_ratio_Gold)
max_hedge_gold <- max(Hedge_ratio_Gold)

mean_hedge_sp <- mean(Hedge_ratio_SP500)
stdev_hedge_sp <- stdev(Hedge_ratio_SP500)
min_hedge_sp <- min(Hedge_ratio_SP500)
max_hedge_sp <- max (Hedge_ratio_SP500)

Hedge_Summary_Stat <- data.frame(
  Measure = c("Gold/SP500", "SP500/Gold"),
  Mean = c(mean_hedge_gold, mean_hedge_sp),
  St.Dev = c(stdev_hedge_gold, stdev_hedge_sp),
  Min = c(min_hedge_gold , min_hedge_sp),
  Max = c(max_hedge_gold ,max_hedge_sp)
)
print(Hedge_Summary_Stat)

Portfolio_weightx = (Cov_SP500 - Cov_uni) / (Cov_Gold - 2*Cov_uni + Cov_SP500)
Portfolio_weightx <- pmax(pmin(Portfolio_weightx, 1), -1)
PW_mean <- mean(Portfolio_weightx)
PW_stdev <- stdev(Portfolio_weightx)
PW_min <- min(Portfolio_weightx)
PW_max <- max (Portfolio_weightx)


Portfolio_weight_Sum <- data.frame (
  Measure = "Gold/SP500",
  mean = PW_mean,
  St.Dev = PW_stdev,
  Min = PW_min,
  Max =PW_max
)

print (Portfolio_weight_Sum)

sjekk <- data.frame(reg_x$Date, Portfolio_weightx, reg_x$GPR)
sjekk2 <- data.frame(reg_x$Date, Hedge_ratio_Gold)
sjekk3 <- data.frame(reg_x$Date, Hedge_ratio_SP500)


p1 <- ggplot(sjekk, aes( x = reg_x$Date, y = Portfolio_weightx)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Portfolio Weights", 
       title = "Portfolio weights Gold/SP500") +
  theme_minimal()

ggplot(sjekk2, aes( x = reg_x.Date, y = Hedge_ratio_Gold)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Hedge ratio", 
       title = "Hedge ratio Gold/SP500") +
  theme_minimal()

ggplot(sjekk3, aes( x = reg_x.Date, y = Hedge_ratio_SP500)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Hedge ratio", 
       title = "Hedge ratio SP500/Gold") +
  theme_minimal()



#################################################################
####################### Crude & OBX #############################
#################################################################

Cov_Crude <- volatilityy[,1]
Cov_OBX <- volatilityy [,2]
Cov_uni <- abs(covy[1,2,])

Hedge_ratio_Crude <- Cov_uni / Cov_OBX
Hedge_ratio_OBX <- Cov_uni / Cov_Crude

mean_hedge_Crude <- mean(Hedge_ratio_Crude)
stdev_hedge_Crude <- stdev(Hedge_ratio_Crude)
min_hedge_Crude <- min(Hedge_ratio_Crude)
max_hedge_Crude <- max(Hedge_ratio_Crude)

mean_hedge_OBX <- mean(Hedge_ratio_OBX)
stdev_hedge_OBX <- stdev(Hedge_ratio_OBX)
min_hedge_OBX <- min(Hedge_ratio_OBX)
max_hedge_OBX <- max (Hedge_ratio_OBX)

Hedge_Summary_Stat <- data.frame(
  Measure = c("Crude/OBX", "OBX/Crude"),
  Mean = c(mean_hedge_Crude, mean_hedge_OBX),
  St.Dev = c(stdev_hedge_Crude, stdev_hedge_OBX),
  Min = c(min_hedge_Crude, min_hedge_OBX),
  Max = c(max_hedge_Crude,max_hedge_OBX)
)
print(Hedge_Summary_Stat)

Portfolio_weighty = (Cov_OBX - Cov_uni) / (Cov_Crude - 2*Cov_uni + Cov_OBX)
Portfolio_weighty <- pmax(pmin(Portfolio_weighty, 1), 0)
PW_mean <- mean(Portfolio_weighty)
PW_stdev <- stdev(Portfolio_weighty)
PW_min <- min(Portfolio_weighty)
PW_max <- max (Portfolio_weighty)


Portfolio_weight_Sum <- data.frame (
  Measure = "Crude/OBX",
  mean = PW_mean,
  St.Dev = PW_stdev,
  Min = PW_min,
  Max =PW_max
)

print (Portfolio_weight_Sum)

sjekk <- data.frame(reg_y$Date,Portfolio_weighty)
sjekk2 <- data.frame(reg_y$Date, Hedge_ratio_Crude)
sjekk3 <- data.frame(reg_y$Date, Hedge_ratio_OBX)

p2 <- ggplot(sjekk, aes( x = reg_y$Date, y = Portfolio_weighty)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Portfolio Weights", 
       title = "Portfolio weights Crude/OBX") +
  theme_minimal()

ggplot(sjekk2, aes( x = reg_y.Date, y = Hedge_ratio_Crude)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Hedge ratio", 
       title = "Hedge ratio Crude/OBX") +
  theme_minimal()

ggplot(sjekk3, aes( x = reg_y.Date, y = Hedge_ratio_OBX)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Hedge ratio", 
       title = "Hedge ratio OBX/Crude") +
  theme_minimal()

#################################################################
####################### Gold & OBX ##############################
#################################################################

Cov_Gold <- volatilityz[,1]
Cov_OBX <- volatilityz [,2]
Cov_uni <- abs(covz[1,2,])

Hedge_ratio_Gold <- Cov_uni / Cov_OBX
Hedge_ratio_OBX <- Cov_uni / Cov_Gold

mean_hedge_Gold <- mean(Hedge_ratio_Gold)
stdev_hedge_Gold <- stdev(Hedge_ratio_Gold)
min_hedge_Gold <- min(Hedge_ratio_Gold)
max_hedge_Gold <- max(Hedge_ratio_Gold)

mean_hedge_OBX <- mean(Hedge_ratio_OBX)
stdev_hedge_OBX <- stdev(Hedge_ratio_OBX)
min_hedge_OBX <- min(Hedge_ratio_OBX)
max_hedge_OBX <- max (Hedge_ratio_OBX)

Hedge_Summary_Stat <- data.frame(
  Measure = c("Gold/OBX", "OBX/Gold"),
  Mean = c(mean_hedge_Gold, mean_hedge_OBX),
  St.Dev = c(stdev_hedge_Gold, stdev_hedge_OBX),
  Min = c(min_hedge_Gold, min_hedge_OBX),
  Max = c(max_hedge_Gold,max_hedge_OBX)
)
print(Hedge_Summary_Stat)

Portfolio_weightz = (Cov_OBX - Cov_uni) / (Cov_Gold - 2*Cov_uni + Cov_OBX)
Portfolio_weightz <- pmax(pmin(Portfolio_weightz, 1), 0)
PW_mean <- mean(Portfolio_weightz)
PW_stdev <- stdev(Portfolio_weightz)
PW_min <- min(Portfolio_weightz)
PW_max <- max (Portfolio_weightz)


Portfolio_weight_Sum <- data.frame (
  Measure = "Gold/OBX",
  mean = PW_mean,
  St.Dev = PW_stdev,
  Min = PW_min,
  Max =PW_max
)

print (Portfolio_weight_Sum)

sjekk <- data.frame(reg_z$Date,Portfolio_weightz)
sjekk2 <- data.frame(reg_z$Date, Hedge_ratio_Gold)
sjekk3 <- data.frame(reg_z$Date, Hedge_ratio_OBX)

p3 <- ggplot(sjekk, aes( x = reg_z$Date, y = Portfolio_weightz)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Portfolio Weights", 
       title = "Portfolio weights Gold/OBX") +
  theme_minimal()

ggplot(sjekk2, aes( x = reg_z.Date, y = Hedge_ratio_Gold)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Hedge ratio", 
       title = "Hedge ratio Gold/OBX") +
  theme_minimal()

ggplot(sjekk3, aes( x = reg_z.Date, y = Hedge_ratio_OBX)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Hedge ratio", 
       title = "Hedge ratio OBX/Gold") +
  theme_minimal()


#################################################################
####################### Crude & SP ##############################
#################################################################

Cov_Crude <- volatilityw[,1]
Cov_SP <- volatilityw [,2]
Cov_uni <- abs(covw[1,2,])

Hedge_ratio_Crude <- Cov_uni / Cov_SP500
Hedge_ratio_SP <- Cov_uni / Cov_Crude

mean_hedge_Crude <- mean(Hedge_ratio_Crude)
stdev_hedge_Crude <- stdev(Hedge_ratio_Crude)
min_hedge_Crude <- min(Hedge_ratio_Crude)
max_hedge_Crude <- max(Hedge_ratio_Crude)

mean_hedge_SP <- mean(Hedge_ratio_SP)
stdev_hedge_SP <- stdev(Hedge_ratio_SP)
min_hedge_SP <- min(Hedge_ratio_SP)
max_hedge_SP <- max (Hedge_ratio_SP)

Hedge_Summary_Stat <- data.frame(
  Measure = c("Crude/SP", "SP/Crude"),
  Mean = c(mean_hedge_Crude, mean_hedge_SP),
  St.Dev = c(stdev_hedge_Crude, stdev_hedge_SP),
  Min = c(min_hedge_Crude, min_hedge_SP),
  Max = c(max_hedge_Crude,max_hedge_SP)
)
print(Hedge_Summary_Stat)

Portfolio_weightw = (Cov_SP500 - Cov_uni) / (Cov_Crude - 2*Cov_uni + Cov_SP500)
Portfolio_weightw <- pmax(pmin(Portfolio_weightw, 1), 0)
PW_mean <- mean(Portfolio_weightw)
PW_stdev <- stdev(Portfolio_weightw)
PW_min <- min(Portfolio_weightw)
PW_max <- max (Portfolio_weightw)


Portfolio_weight_Sum <- data.frame (
  Measure = "Crude/SP",
  mean = PW_mean,
  St.Dev = PW_stdev,
  Min = PW_min,
  Max =PW_max
)

print (Portfolio_weight_Sum)

sjekk <- data.frame(reg_w$Date,Portfolio_weightw)
sjekk2 <- data.frame(reg_w$Date, Hedge_ratio_Crude)
sjekk3 <- data.frame(reg_w$Date, Hedge_ratio_SP)

p4 <- ggplot(sjekk, aes( x = reg_w$Date, y = Portfolio_weightw)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Portfolio Weights", 
       title = "Portfolio weights Crude/SP500") +
  theme_minimal()

ggplot(sjekk2, aes( x = reg_w.Date, y = Hedge_ratio_Crude)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Hedge ratio", 
       title = "Hedge ratio Crude/SP500") +
  theme_minimal()

ggplot(sjekk3, aes( x = reg_w.Date, y = Hedge_ratio_SP)) +
  geom_line(color = "black") +
  labs(x = "Time", y = "Hedge ratio", 
       title = "Hedge ratio SP500/Crude") +
  theme_minimal()

grid.arrange (p1,p2,p3,p4, ncol =2, nrow =2)

